<?php
/**
 * Created by PhpStorm.
 * User: proess
 * Date: 2/06/23
 * Time: 16:20 PM
 */
namespace IMATHUZH\Qfq\Core\Form;

use CodeException;
use DbException;
use IMATHUZH\Qfq\Core\Form\FormElement\AbstractFormElement;
use IMATHUZH\Qfq\Core\Form\FormElement\CheckboxFormElement;
use IMATHUZH\Qfq\Core\Form\FormElement\DatetimeFormElement;
use IMATHUZH\Qfq\Core\Form\FormElement\EditorFormElement;
use IMATHUZH\Qfq\Core\Form\FormElement\ExtraFormElement;
use IMATHUZH\Qfq\Core\Form\FormElement\FieldsetFormElement;
use IMATHUZH\Qfq\Core\Form\FormElement\ImageCutFormElement;
use IMATHUZH\Qfq\Core\Form\FormElement\InputFormElement;
use IMATHUZH\Qfq\Core\Form\FormElement\NoteFormElement;
use IMATHUZH\Qfq\Core\Form\FormElement\PillFormElement;
use IMATHUZH\Qfq\Core\Form\FormElement\RadioButtonFormElement;
use IMATHUZH\Qfq\Core\Form\FormElement\SelectFormElement;
use IMATHUZH\Qfq\Core\Form\FormElement\SubrecordFormElement;
use IMATHUZH\Qfq\Core\Helper\Support;
use UserFormException;
use UserReportException;

/**
 * Class FeFactory
 * @package qfq
 */
class FeFactory {
    /**
     * Creates and returns an instance of a concrete class that extends the AbstractFormElement class, based on the given feType.
     *
     * @param string $recordId The record ID.
     * @param mixed $value The value of the form element.
     * @param string $column The column name.
     * @param string $table The table name.
     * @param string $feType The form element type.
     *
     * @throws UserFormException
     * @throws CodeException
     * @throws UserReportException
     * @throws DbException
     *
     * @return AbstractFormElement Instance of concrete FormElement class.
     */
    public static function createFe(string $recordId, string $value, string $column
        , string $table, string $feType, string $dbIndex):AbstractFormElement {

        // Define needed default values
        $formElement = array(FE_NAME => $column,
            FE_MODE_SQL => '',
            FE_ENCODE => 'specialchar',
            FE_CHECK_TYPE => 'auto',
            FE_MODE => 'show',
            FE_HTML_ID => $column . '-' . $table . '-' . $recordId,
        );

        $htmlFormElementName = $column . '-' . $table . '-' . $recordId;

        // Design Pattern: Factory
        // Instantiate the formElement object based on the feType
        switch ($feType){
            case FE_TYPE_TEXT:
                $formElement[FE_TYPE] = 'text';
                $formElement[FE_SIZE] = '1,0,1';

                #todo would ne nice to not call setFeDefault() for every type, only once would be better
                $formElement = Support::setFeDefaults($formElement);

                $fe = new FormElementInput($formElement, $value, $htmlFormElementName, $dbIndex);
                break;

            case FE_TYPE_SELECT:
                $formElement[FE_TYPE] = FE_TYPE_SELECT;
                break;

            case FE_TYPE_UPLOAD:
                $formElement[FE_TYPE] = FE_TYPE_UPLOAD;
                break;

        }

        return $fe;

    }



    /**
     * Creates and returns an instance of a concrete class that extends the AbstractFormElement class, based on the given feType.
     *
     * @param string $formElementSpec The record ID.
     * @param ?Form $form Object representing the form, to which the FormElement belongs. Can be null, i.e. for inline-edit elements.
     * @throws UserFormException
     * @throws CodeException
     * @throws UserReportException
     * @throws DbException
     *
     * @return AbstractFormElement Instance of concrete FormElement class.
     */
    public static function createFormElement(array $formElementSpec, ?Form $form):AbstractFormElement {
        // TODO: examine FE Spec, extract necessary infos to call self::createFE
        //$htmlFormElementName = $column . '-' . $table . '-' . $recordId;
        $feType = $formElementSpec['type'];

        $formElement = null;

        switch ($feType){
            case FE_TYPE_PILL:
                $formElement = new PillFormElement($formElementSpec, $form);
                break;
            case FE_TYPE_FIELDSET:
                $formElement = new FieldsetFormElement($formElementSpec, $form);
                break;
            case FE_TYPE_TEXT:
                $formElement = new InputFormElement($formElementSpec, $form);
                break;
            case FE_TYPE_SUBRECORD:
                $formElement = new SubrecordFormElement($formElementSpec, $form);
                break;
            case FE_TYPE_CHECKBOX:
                $formElement = new CheckboxFormElement($formElementSpec, $form);
                break;
            case FE_TYPE_RADIO:
                $formElement = new RadioButtonFormElement($formElementSpec, $form);
                break;
            case FE_TYPE_SELECT:
                $formElement = new SelectFormElement($formElementSpec, $form);
                break;
            case FE_TYPE_NOTE:
                $formElement = new NoteFormElement($formElementSpec, $form);
                break;
            case FE_TYPE_EDITOR:
                $formElement = new EditorFormElement($formElementSpec, $form);
                break;
            case FE_TYPE_DATETIME:
            case FE_TYPE_DATE:
            case FE_TYPE_TIME:
                $formElement = new DatetimeFormElement($formElementSpec, $form);
                break;
            case FE_TYPE_IMAGE_CUT:
                $formElement = new ImageCutFormElement($formElementSpec, $form);
                break;
            case FE_TYPE_EXTRA:
                $formElement = new ExtraFormElement($formElementSpec, $form);
                break;
            default:
                return new InputFormElement($formElementSpec); // TODO: dont.
        }
        return $formElement;
    }

}